////////////////////////////////////////////////////////////////
// Capella 2002, Version 4.0 - Dokumentation des Dateiformats
// LowLevel.cpp

#include "CapRead.h"

static ifstream  in;
static HINSTANCE hInst;

////////////////////////////////////////////////////////////////
// Testrahmen (WinMain)

int CALLBACK WinMain (HINSTANCE hInstance,
                      HINSTANCE hPrevInstance,
                      LPSTR     lpszCmdParam,
                      int       nCmdShow)
{
  hInst = hInstance;
  // Tests fuer Dateiformat
  // Test der Typgroessen
  assert (CHAR_MAX == SCHAR_MAX); // char ist signed char
  assert (sizeof(bool)  == 1);
  assert (sizeof(BYTE)  == 1);
  assert (sizeof(char)  == 1);
  assert (sizeof(WORD)  == 2);
  assert (sizeof(short) == 2);
  assert (sizeof(DWORD) == 4);
  assert (sizeof(int)   == 4);
  // Test der Intel-Zahlanordnung (niedriges Byte zuerst)
  union {
    DWORD dw;
    BYTE  b[4];
  };
  dw = 1;
  assert (b[0] == 1);

  // Datei waehlen
  char fileName[MAX_PATH] = "";
  OPENFILENAME of;
  memset (&of, 0, sizeof (OPENFILENAME));
  of.lStructSize = sizeof(OPENFILENAME);
  of.hInstance   = hInst;
  of.lpstrFilter = "capella-Dateien (*.cap)\0*.cap\0";
  of.lpstrFile   = (LPSTR)fileName; // mit vollstaendigem Pfad
  of.nMaxFile    = MAX_PATH;
  of.Flags = OFN_FILEMUSTEXIST;
  BOOL ok = GetOpenFileName(&of);
  assert (ok && fileName != NULL);
  in.open (fileName, ios::in | ios::binary);
  assert (in.is_open());

  Score score;

  in.close();

  return 0;
}

////////////////////////////////////////////////////////////////

Byte::Byte () {
  in.read ((char*)&value, 1);
}

Word::Word() {
  in.read ((char*)&value, 2);
}

DWord::DWord() {
  in.read ((char*)&value, 4);
}

Char::Char() {
  in.read (&value, 1);
}

Bool::Bool() {
  in.read ((char*)&value, 1);
}

Integer::Integer() {
  // je nach Groesse 1, 3 oder 5 Bytes
  char c;
  int n;
  in.read ((char*)&c, 1);
  if (c == SCHAR_MIN) {        // -128
    short s;
    in.read ((char*)&s, 2);
    n = (int) s;
  }
  else if (c == SCHAR_MAX)     // 127
    in.read ((char*)&n, 4);
  else
    n = (int) c;
  value = n;
}

Unsigned::Unsigned() {
  // je nach Groesse 1, 3 oder 5 Bytes
  value = 0;
  in.read ((char*)&value, 1);
  if (value >= 254) { // niedriges Byte vorn (Intel)
    if (value == 254) in.read ((char*)&value, 2);
    else          in.read ((char*)&value, 4);
  }
}

CharArray::CharArray (int n) {
  buffer = new char[n];
  in.read (buffer, n);
}

CharArray::~CharArray() {
  delete[] buffer;
}

String::String() {
  Unsigned lg;
  buffer = new char [lg+1];
  in.read (buffer, lg);
  buffer[lg] = '\0';
}

String::~String() {
  delete[] buffer;
}

Color::Color() {
  static COLORREF stdColor[16] = {
    0x000000, // schwarz   
    0x000080, // dunkelrot 
    0x008000, // dunkelgrn
    0x008080, // ocker     
    0x800000, // dunkelblau
    0x800080, // purpurrot 
    0x808000, // blaugrn  
    0x808080, // grau      
    0xC0C0C0, // hellgrau  
    0x0000FF, // rot       
    0x00FF00, // grn      
    0x00FFFF, // gelb      
    0xFF0000, // blau      
    0xFF00FF, // lila      
    0xFFFF00, // aquamarin 
    0xFFFFFF  // wei      
  };

  Byte b;
  if (b < 16)
    value = stdColor[b];
  else {
    assert (b == 255);
    value = 0;
    in.read ((char*)&value, 3);
  }
}

DEFINITION (ExtraInfo) {            // Zusatzinformationen fr knftige Erweiterungen
  Unsigned      nExtra;             // bei capella 2000 derzeit = 0
                                    // sptere Versionen knnen hier zustzliche
                                    // Informationen ablegen, die von capella 2000
                                    // ignoriert werden (Abwrtskompatibilitt!).
                                    // Ein Programm, das capella-Dateien liest, mu davon
                                    // ausgehen, da nExtra auch > 0 sein kann!
  for (int i=0; i<nExtra; i++)
    Byte        extra;              // beim Lesen extra-Bytes ignorieren
}
